/* $Id: HestonPricer.tm 3092 2010-09-29 11:54:54Z kletz $ */

#include "HestonPricer.h"

#undef double

void COSPricerWrapper(
		double s0,
		int* maturities, long maturitiesLen,
		double* strikes, long strikesLen,
		double r, double rf,
		double* hestonParams, long hestonParamsLen) {

	int numberofoptions = (strikesLen < maturitiesLen)? strikesLen : maturitiesLen;
	float* prices=new float[numberofoptions];
	
	float* str = new float[numberofoptions];
	float* hp = new float[hestonParamsLen];
	for (int i = 0; i<numberofoptions; i++) {
		str[i] = (float)strikes[i];
	}
	for (int i = 0; i<hestonParamsLen; i++) {
		hp[i] = (float)hestonParams[i];
	}
	
	int result = COSPricer(s0,numberofoptions,str, maturities, r, rf, hp, prices);

	double* pr = new double[numberofoptions];
	for (int i = 0; i<numberofoptions; i++) {
		pr[i] = (double)prices[i];
	}

	if (result == 1)
		MLPutRealList(stdlink, pr, numberofoptions);
	else
		MLPutInteger(stdlink, result);

	delete [] str;
	delete [] hp;
	delete [] prices;
	delete [] pr;
}

void COSPricerWrapperGPU(
		double s0,
		int* maturities, long maturitiesLen,
		double* strikes, long strikesLen,
		double r, double rf,
		double* hestonParams, long hestonParamsLen) {

	int numberofoptions = (strikesLen < maturitiesLen)? strikesLen : maturitiesLen;
	float* prices=new float[numberofoptions];

	float* str = new float[numberofoptions];
	float* hp = new float[hestonParamsLen];
	for (int i = 0; i<numberofoptions; i++) {
		str[i] = (float)strikes[i];
	}
	for (int i = 0; i<hestonParamsLen; i++) {
		hp[i] = (float)hestonParams[i];
	}

	int result = COSPricerGPU(s0,numberofoptions,str, maturities, r, rf, hp, prices);

	double* pr = new double[numberofoptions];
	for (int i = 0; i<numberofoptions; i++) {
		pr[i] = (double)prices[i];
	}

	if (result == 1)
		MLPutRealList(stdlink, pr, numberofoptions);
	else
		MLPutInteger(stdlink, result);

	delete [] str;
	delete [] hp;
	delete [] prices;
	delete [] pr;
}

:Begin:
:Function:       COSPricerWrapper
:Pattern:        COSPricer[s0_Real, scenarios: {{_Integer, _Real} ..}, r_Real, rf_Real, hestonParams: { __Real }]
:Arguments:      { s0, scenarios[[All, 1]], scenarios[[All, 2]], r, rf, hestonParams }
:ArgumentTypes:  { Real, IntegerList, RealList, Real, Real, RealList }
:ReturnType:     Manual
:End:

:Evaluate: COSPricer::usage = "COSPricer[...] valuates a list of equity options according to the given Heston model."

:Begin:
:Function:       COSPricerWrapperGPU
:Pattern:        COSPricerGPU[s0_Real, scenarios: {{_Integer, _Real} ..}, r_Real, rf_Real, hestonParams: { __Real }]
:Arguments:      { s0, scenarios[[All, 1]], scenarios[[All, 2]], r, rf, hestonParams }
:ArgumentTypes:  { Real, IntegerList, RealList, Real, Real, RealList }
:ReturnType:     Manual
:End:

:Evaluate: COSPricerGPU::usage = "COSPricerGPU[...] valuates a list of equity options according to the given Heston model by using the GPU."
