(* Content-type: application/mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 6.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       145,          7]
NotebookDataLength[     48613,       1185]
NotebookOptionsPosition[     47419,       1141]
NotebookOutlinePosition[     48066,       1166]
CellTagsIndexPosition[     48023,       1163]
WindowTitle->Volatility Surface in the Heston Model - Source
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Volatility Surface in the Heston Model", "Section",
 CellFrameColor->RGBColor[
  0.6449835965514611, 0.758632791638056, 0.2516823071641108],
 FontColor->RGBColor[
  0.6449835965514611, 0.758632791638056, 0.2516823071641108]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Hhat", "[", 
   RowBox[{
   "k_", ",", "V_", ",", "\[Tau]_", ",", "\[Theta]_", ",", "\[Xi]_", ",", 
    "\[Omega]_", ",", "\[Rho]_"}], "]"}], ":=", 
  RowBox[{"Module", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
     "f1", ",", "f2", ",", "d", ",", "g", ",", "h", ",", "ttheta", ",", "t", 
      ",", "tomega", ",", "tc"}], "}"}], ",", 
    RowBox[{
     RowBox[{"c", "=", 
      RowBox[{
       FractionBox["1", "2"], 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["k", "2"], "-", 
         RowBox[{"\[ImaginaryI]", " ", "k"}]}], ")"}]}]}], ";", 
     RowBox[{"t", "=", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox["\[Xi]", "2"], " ", "\[Tau]"}]}], ";", 
     RowBox[{"tomega", "=", 
      RowBox[{
       FractionBox["2", 
        SuperscriptBox["\[Xi]", "2"]], " ", "\[Omega]"}]}], ";", 
     RowBox[{"tc", "=", 
      RowBox[{
       FractionBox["2", 
        SuperscriptBox["\[Xi]", "2"]], " ", "c"}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"ttheta", "=", 
      RowBox[{
       FractionBox["2", 
        SuperscriptBox["\[Xi]", "2"]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"\[ImaginaryI]", " ", "k", " ", "\[Rho]", " ", "\[Xi]"}], 
         "+", "\[Theta]"}], ")"}]}]}], ";", 
     RowBox[{"d", "=", 
      SqrtBox[
       RowBox[{
        SuperscriptBox["ttheta", "2"], "+", 
        RowBox[{"4", " ", "tc"}]}]]}], ";", 
     RowBox[{"g", "=", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"(", 
        RowBox[{"ttheta", "+", "d"}], ")"}]}]}], ";", 
     RowBox[{"h", "=", 
      FractionBox[
       RowBox[{"ttheta", "+", "d"}], 
       RowBox[{"ttheta", "-", "d"}]]}], ";", "\[IndentingNewLine]", 
     RowBox[{"f1", "=", 
      RowBox[{"tomega", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"t", " ", "g"}], "-", 
         RowBox[{"Log", "[", 
          FractionBox[
           RowBox[{"1", "-", 
            RowBox[{"h", " ", 
             RowBox[{"Exp", "[", 
              RowBox[{"d", " ", "t"}], "]"}]}]}], 
           RowBox[{"1", "-", "h"}]], "]"}]}], ")"}]}]}], ";", 
     RowBox[{"f2", "=", 
      RowBox[{"g", " ", 
       RowBox[{"(", 
        FractionBox[
         RowBox[{"1", "-", 
          RowBox[{"Exp", "[", 
           RowBox[{"d", " ", "t"}], "]"}]}], 
         RowBox[{"1", "-", 
          RowBox[{"h", " ", 
           RowBox[{"Exp", "[", 
            RowBox[{"d", " ", "t"}], "]"}]}]}]], ")"}]}]}], ";", 
     "\[IndentingNewLine]", 
     RowBox[{"Exp", "[", 
      RowBox[{"f1", "+", 
       RowBox[{"f2", " ", "V"}]}], "]"}]}]}], "]"}]}], "\n", 
 RowBox[{
  RowBox[{"kim", "=", "0.6"}], ";", 
  RowBox[{"\[CapitalLambda]", "=", "125"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"HestonCallLewis", "[", 
    RowBox[{
    "S_", ",", "K_", ",", "\[Tau]_", ",", "r_", ",", "d_", ",", "V0_", ",", 
     "MR_", ",", "\[Rho]_", ",", "Vinf_", ",", "VolVol_"}], "]"}], ":=", 
   RowBox[{"Module", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
      "Integral", ",", "X", ",", "\[Theta]", ",", "\[Xi]", ",", "\[Omega]", 
       ",", "V", ",", "k"}], "}"}], ",", 
     RowBox[{
      RowBox[{"X", "=", 
       RowBox[{
        RowBox[{"Log", "[", 
         FractionBox["S", "K"], "]"}], "+", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"r", "-", "d"}], ")"}], "\[Tau]"}]}]}], ";", 
      RowBox[{"k", "=", 
       RowBox[{"kre", "+", 
        RowBox[{"\[ImaginaryI]", " ", "kim"}]}]}], ";", "\[IndentingNewLine]", 
      RowBox[{"Integral", "=", 
       RowBox[{
        FractionBox["1", "\[Pi]"], " ", 
        RowBox[{"NIntegrate", "[", 
         RowBox[{
          RowBox[{"Re", "[", 
           RowBox[{
            RowBox[{"Exp", "[", 
             RowBox[{
              RowBox[{"-", "\[ImaginaryI]"}], " ", "k", " ", "X"}], "]"}], 
            " ", 
            RowBox[{"(", 
             FractionBox[
              RowBox[{"Hhat", "[", 
               RowBox[{
               "k", ",", "V0", ",", "\[Tau]", ",", "MR", ",", "VolVol", ",", 
                RowBox[{"Vinf", " ", "MR"}], ",", "\[Rho]"}], "]"}], 
              RowBox[{
               SuperscriptBox["k", "2"], "-", 
               RowBox[{"\[ImaginaryI]", " ", "k"}]}]], ")"}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"kre", ",", "0", ",", "\[CapitalLambda]"}], "}"}], ",", 
          RowBox[{"Method", "\[Rule]", 
           RowBox[{"{", 
            RowBox[{"\"\<LobattoKronrodRule\>\"", ",", 
             RowBox[{"\"\<Points\>\"", "\[Rule]", "25"}]}], "}"}]}], ",", 
          RowBox[{"PrecisionGoal", "\[Rule]", "4"}]}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"S", " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "d"}], " ", "\[Tau]"}], "]"}]}], "-", 
       RowBox[{"K", " ", 
        RowBox[{"Exp", "[", 
         RowBox[{
          RowBox[{"-", "r"}], " ", "\[Tau]"}], "]"}], " ", "Integral"}]}]}]}],
     "]"}]}], "\n"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Moneyness", "=", 
   RowBox[{"{", 
    RowBox[{
    ".6", ",", ".75", ",", ".9", ",", "1", ",", "1.1", ",", "1.25", ",", 
     "1.4"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"Tenors", "=", 
   RowBox[{"{", 
    RowBox[{
    "1", ",", "3", ",", "6", ",", "12", ",", "18", ",", "24", ",", "30", ",", 
     "36"}], "}"}]}], ";"}], "\n", 
 RowBox[{
  RowBox[{"m", "=", 
   RowBox[{"Length", "[", "Moneyness", "]"}]}], ";", 
  RowBox[{"t", "=", 
   RowBox[{"Length", "[", "Tenors", "]"}]}], ";", 
  RowBox[{"size", "=", 
   RowBox[{"t", " ", "m"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"S", "=", "100"}], ";", 
  RowBox[{"r", "=", "0.03"}], ";", 
  RowBox[{"d", "=", "0."}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Ncdf", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"{", 
       RowBox[{"x", ",", "_Real"}], "}"}], "}"}], ",", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Erf", "[", 
         RowBox[{"x", "/", 
          SqrtBox["2"]}], "]"}], "+", "1"}], ")"}], "/", "2"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"BSCall", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"S", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"K", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"\[Sigma]", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"T", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"r", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"d", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"d1", ",", "d2"}], "}"}], ",", 
       RowBox[{
        RowBox[{"d1", "=", 
         FractionBox[
          RowBox[{
           RowBox[{"Log", "[", 
            FractionBox["S", "K"], "]"}], "+", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{"r", "-", "d"}], ")"}], " ", "T"}], "+", 
           RowBox[{
            SuperscriptBox["\[Sigma]", "2"], " ", 
            RowBox[{"T", "/", "2"}]}]}], 
          RowBox[{"\[Sigma]", 
           SqrtBox["T"]}]]}], ";", 
        RowBox[{"d2", "=", 
         RowBox[{"d1", "-", 
          RowBox[{"\[Sigma]", " ", 
           SqrtBox["T"]}]}]}], ";", "\[IndentingNewLine]", 
        RowBox[{
         RowBox[{"S", " ", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "d"}], " ", "T"}]], " ", 
          RowBox[{"Ncdf", "[", "d1", "]"}]}], "-", 
         RowBox[{"K", " ", 
          SuperscriptBox["\[ExponentialE]", 
           RowBox[{
            RowBox[{"-", "r"}], " ", "T"}]], " ", 
          RowBox[{"Ncdf", "[", "d2", "]"}]}]}]}]}], "]"}]}], "]"}]}], 
  ";"}], "\n", 
 RowBox[{
  RowBox[{"ImpliedVolCall", "=", 
   RowBox[{"Compile", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"p", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"S", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"K", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"T", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"r", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"d", ",", "_Real"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"precision", ",", "_Real"}], "}"}]}], "}"}], ",", 
     RowBox[{"Module", "[", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
        "Vol0", ",", "Vol1", ",", "Vol2", ",", "Price0", ",", "Price1", ",", 
         "Price2"}], "}"}], ",", 
       RowBox[{
        RowBox[{"Vol2", "=", "0.8"}], ";", 
        RowBox[{"Vol0", "=", "0."}], ";", 
        RowBox[{"Price0", "=", 
         RowBox[{
          RowBox[{"S", " ", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "d"}], " ", "T"}], "]"}]}], "-", 
          RowBox[{"K", " ", 
           RowBox[{"Exp", "[", 
            RowBox[{
             RowBox[{"-", "r"}], " ", "T"}], "]"}]}]}]}], ";", 
        "\[IndentingNewLine]", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"Price0", "<", "0"}], ",", 
          RowBox[{"Price0", "=", "0."}]}], "]"}], ";", "\[IndentingNewLine]", 
        
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"p", "<", "Price0"}], ",", "0.000001", ",", 
          RowBox[{
           RowBox[{"Vol1", "=", "Vol2"}], ";", 
           RowBox[{"Price2", "=", 
            RowBox[{"BSCall", "[", 
             RowBox[{
             "S", ",", "K", ",", "Vol1", ",", "T", ",", "r", ",", "d"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"While", "[", 
            RowBox[{
             RowBox[{"Price2", "<", "p"}], ",", 
             RowBox[{
              RowBox[{"Price2", "=", 
               RowBox[{"BSCall", "[", 
                RowBox[{
                "S", ",", "K", ",", "Vol1", ",", "T", ",", "r", ",", "d"}], 
                "]"}]}], ";", 
              RowBox[{"Vol1", "=", 
               RowBox[{"2", " ", "Vol1"}]}], ";"}]}], "]"}], ";", 
           "\[IndentingNewLine]", 
           RowBox[{"Vol2", "=", "Vol1"}], ";", 
           RowBox[{"Price1", "=", 
            RowBox[{"BSCall", "[", 
             RowBox[{
             "S", ",", "K", ",", "Vol1", ",", "T", ",", "r", ",", "d"}], 
             "]"}]}], ";", "\[IndentingNewLine]", 
           RowBox[{"While", "[", 
            RowBox[{
             RowBox[{
              RowBox[{"Abs", "[", 
               RowBox[{"Price1", "-", "p"}], "]"}], ">", "precision"}], ",", 
             RowBox[{
              RowBox[{"Vol1", "=", 
               RowBox[{
                RowBox[{"(", 
                 RowBox[{"Vol0", "+", "Vol2"}], ")"}], "/", "2."}]}], ";", 
              RowBox[{"Price1", "=", 
               RowBox[{"BSCall", "[", 
                RowBox[{
                "S", ",", "K", ",", "Vol1", ",", "T", ",", "r", ",", "d"}], 
                "]"}]}], ";", "\[IndentingNewLine]", 
              RowBox[{"If", "[", 
               RowBox[{
                RowBox[{"Price1", "<", "p"}], ",", 
                RowBox[{"Vol0", "=", "Vol1"}], ",", 
                RowBox[{"Vol2", "=", "Vol1"}]}], "]"}], ";"}]}], "]"}], ";", 
           "\[IndentingNewLine]", "Vol1"}]}], "]"}]}]}], "]"}]}], "]"}]}], 
  ";"}]}], "Input",
 InitializationCell->True,
 CellChangeTimes->{
  3.35696210375764*^9, {3.418769162171875*^9, 3.4187694736875*^9}, {
   3.418769506734375*^9, 3.418769636265625*^9}, {3.4187759466875*^9, 
   3.418775946734375*^9}, {3.418776022390625*^9, 3.41877602328125*^9}, {
   3.4196231893673773`*^9, 3.4196233427736273`*^9}, {3.4196234460236273`*^9, 
   3.4196234619298773`*^9}, {3.4196247303673773`*^9, 3.4196247308986273`*^9}},
 
 CellID->92223448],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Manipulate", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Quiet", "@", 
    RowBox[{"Module", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
       "i", ",", "j", ",", "x", ",", "k", ",", "T", ",", "Price", ",", "pr", 
        ",", "Volatility", ",", "vol"}], "}"}], ",", "\[IndentingNewLine]", 
      RowBox[{
       RowBox[{"Price", "=", 
        RowBox[{"Array", "[", 
         RowBox[{"pr", ",", 
          RowBox[{"{", 
           RowBox[{"m", ",", "t"}], "}"}]}], "]"}]}], ";", 
       "\[IndentingNewLine]", 
       RowBox[{"Volatility", "=", 
        RowBox[{"Array", "[", 
         RowBox[{"vol", ",", "size"}], "]"}]}], ";", "\[IndentingNewLine]", 
       RowBox[{"For", "[", 
        RowBox[{
         RowBox[{"i", "=", "1"}], ",", 
         RowBox[{"i", "<", 
          RowBox[{"t", "+", "1"}]}], ",", 
         RowBox[{"i", "++"}], ",", 
         RowBox[{
          RowBox[{"T", "=", 
           RowBox[{
            RowBox[{"Tenors", "[", 
             RowBox[{"[", "i", "]"}], "]"}], "/", "12"}]}], ";", 
          "\[IndentingNewLine]", 
          RowBox[{"For", "[", 
           RowBox[{
            RowBox[{"j", "=", "1"}], ",", 
            RowBox[{"j", "<", 
             RowBox[{"m", "+", "1"}]}], ",", 
            RowBox[{"j", "++"}], ",", 
            RowBox[{
             RowBox[{"k", "=", 
              RowBox[{"S", "/", 
               RowBox[{"Moneyness", "[", 
                RowBox[{"[", "j", "]"}], "]"}]}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{
              RowBox[{"pr", "[", 
               RowBox[{"i", ",", "j"}], "]"}], "=", 
              RowBox[{"HestonCallLewis", "[", 
               RowBox[{"S", ",", "k", ",", "T", ",", "r", ",", "d", ",", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"\[Sigma]0", "/", "100"}], ")"}], "^", "2"}], ",", 
                "MR", ",", "\[Rho]", ",", 
                RowBox[{
                 RowBox[{"(", 
                  RowBox[{"\[Sigma]inf", "/", "100"}], ")"}], "^", "2"}], ",", 
                RowBox[{"VolVol", "/", "100"}]}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"x", "=", 
              RowBox[{"ImpliedVolCall", "[", 
               RowBox[{
                RowBox[{"pr", "[", 
                 RowBox[{"i", ",", "j"}], "]"}], ",", "S", ",", "k", ",", "T",
                 ",", "r", ",", "d", ",", "0.0001"}], "]"}]}], ";", 
             "\[IndentingNewLine]", 
             RowBox[{"If", "[", 
              RowBox[{
               RowBox[{"x", "\[NotEqual]", "0.000001"}], ",", 
               RowBox[{
                RowBox[{"vol", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"i", "-", "1"}], ")"}], " ", "m"}], "+", "j"}], 
                 "]"}], "=", 
                RowBox[{"{", 
                 RowBox[{
                  RowBox[{"Tenors", "[", 
                   RowBox[{"[", "i", "]"}], "]"}], ",", 
                  RowBox[{"Moneyness", "[", 
                   RowBox[{"[", "j", "]"}], "]"}], ",", 
                  RowBox[{"100", " ", "x"}]}], "}"}]}], ",", 
               RowBox[{
                RowBox[{"vol", "[", 
                 RowBox[{
                  RowBox[{
                   RowBox[{"(", 
                    RowBox[{"i", "-", "1"}], ")"}], " ", "m"}], "+", "j"}], 
                 "]"}], "=", 
                RowBox[{"{", 
                 RowBox[{"Null", ",", "Null", ",", "Null"}], "}"}]}]}], "]"}],
              ";"}]}], "]"}], ";"}]}], "]"}], ";", "\[IndentingNewLine]", 
       RowBox[{"ListPlot3D", "[", 
        RowBox[{"Volatility", ",", 
         RowBox[{"PlotRange", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"Full", ",", "Full", ",", "Full"}], "}"}]}], ",", 
         RowBox[{
         "PlotLabel", "\[Rule]", "\"\<Implied Volatility Surface\>\""}], ",", 
         
         RowBox[{"AxesLabel", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{
           "\"\<time to expiry\\nin months\>\"", ",", "\"\<moneyness\>\"", 
            ",", "None"}], "}"}]}], ",", 
         RowBox[{"ImageSize", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"600", ",", "600"}], "}"}]}], ",", 
         RowBox[{"BoxRatios", "\[Rule]", 
          RowBox[{"{", 
           RowBox[{"1", ",", "1", ",", ".6"}], "}"}]}], ",", 
         RowBox[{"ImagePadding", "\[Rule]", "25"}]}], "]"}]}]}], "]"}]}], ",",
    "\[IndentingNewLine]", 
   RowBox[{"Item", "[", "\[IndentingNewLine]", 
    RowBox[{"Grid", "[", 
     RowBox[{
      RowBox[{"{", "\[IndentingNewLine]", 
       RowBox[{
        RowBox[{"{", 
         RowBox[{"\"\<original volatility\>\"", ",", 
          RowBox[{"Manipulator", "[", 
           RowBox[{
            RowBox[{"Dynamic", "[", "\[Sigma]0", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"10", ",", "200", ",", "10"}], "}"}], ",", 
            RowBox[{"ImageSize", "\[Rule]", "Tiny"}]}], "]"}], ",", 
          RowBox[{"Row", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Dynamic", "[", "\[Sigma]0", "]"}], ",", "\"\<%\>\""}], 
            "}"}], "]"}]}], "}"}], ",", 
        RowBox[{"{", 
         RowBox[{"\"\<long-term volatility\>\"", ",", 
          RowBox[{"Manipulator", "[", 
           RowBox[{
            RowBox[{"Dynamic", "[", "\[Sigma]inf", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"10", ",", "200", ",", "10"}], "}"}], ",", 
            RowBox[{"ImageSize", "\[Rule]", "Tiny"}]}], "]"}], ",", 
          RowBox[{"Row", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Dynamic", "[", "\[Sigma]inf", "]"}], ",", "\"\<%\>\""}],
             "}"}], "]"}]}], "}"}], ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"\"\<correlation\>\"", ",", 
          RowBox[{"Manipulator", "[", 
           RowBox[{
            RowBox[{"Dynamic", "[", "\[Rho]", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{
              RowBox[{"-", ".9"}], ",", ".9", ",", ".1"}], "}"}], ",", 
            RowBox[{"ImageSize", "\[Rule]", "Tiny"}]}], "]"}], ",", 
          RowBox[{"Row", "[", 
           RowBox[{"{", 
            RowBox[{"Dynamic", "[", "\[Rho]", "]"}], "}"}], "]"}]}], "}"}], 
        ",", "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"\"\<mean reversion\>\"", ",", 
          RowBox[{"Manipulator", "[", 
           RowBox[{
            RowBox[{"Dynamic", "[", "MR", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{".1", ",", "10", ",", "0.1"}], "}"}], ",", 
            RowBox[{"ImageSize", "\[Rule]", "Tiny"}]}], "]"}], ",", 
          RowBox[{"Dynamic", "[", "MR", "]"}]}], "}"}], ",", 
        "\[IndentingNewLine]", 
        RowBox[{"{", 
         RowBox[{"\"\<volatility of volatility\>\"", ",", 
          RowBox[{"Manipulator", "[", 
           RowBox[{
            RowBox[{"Dynamic", "[", "VolVol", "]"}], ",", 
            RowBox[{"{", 
             RowBox[{"10", ",", "1000", ",", "10"}], "}"}], ",", 
            RowBox[{"ImageSize", "\[Rule]", "Tiny"}]}], "]"}], ",", 
          RowBox[{"Row", "[", 
           RowBox[{"{", 
            RowBox[{
             RowBox[{"Dynamic", "[", "VolVol", "]"}], ",", "\"\<%\>\""}], 
            "}"}], "]"}]}], "}"}]}], "}"}], ",", 
      RowBox[{"Alignment", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{"{", 
         RowBox[{"Right", ",", "Left", ",", "Right"}], "}"}], "}"}]}], ",", 
      RowBox[{"ItemSize", "\[Rule]", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"{", 
          RowBox[{"12", ",", "8", ",", "4"}], "}"}], ",", "Automatic"}], 
        "}"}]}]}], "]"}], "]"}], ",", "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Sigma]0", ",", "40"}], "}"}], ",", "10", ",", "200", ",", 
     "10", ",", 
     RowBox[{"ControlType", "\[Rule]", "None"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Sigma]inf", ",", "20"}], "}"}], ",", "10", ",", "200", ",", 
     "10", ",", 
     RowBox[{"ControlType", "\[Rule]", "None"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"\[Rho]", ",", 
       RowBox[{"-", ".1"}]}], "}"}], ",", 
     RowBox[{"-", ".9"}], ",", ".9", ",", ".1", ",", 
     RowBox[{"ControlType", "\[Rule]", "None"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"MR", ",", "1.5"}], "}"}], ",", ".1", ",", "10", ",", ".1", ",", 
     RowBox[{"ControlType", "\[Rule]", "None"}]}], "}"}], ",", 
   "\[IndentingNewLine]", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"VolVol", ",", "80"}], "}"}], ",", "10", ",", "1000", ",", "10",
      ",", 
     RowBox[{"ControlType", "\[Rule]", "None"}]}], "}"}], ",", 
   RowBox[{"SynchronousUpdating", "\[Rule]", "False"}], ",", 
   RowBox[{"SaveDefinitions", "\[Rule]", "True"}], ",", 
   RowBox[{"ControlPlacement", "\[Rule]", "Left"}], ",", 
   RowBox[{"TrackedSymbols", "\[Rule]", "Manipulate"}]}], "]"}]], "Input",
 CellChangeTimes->{
  3.35696210375764*^9, 3.418768125875*^9, {3.41877563815625*^9, 
   3.418775770328125*^9}, {3.418776032265625*^9, 3.4187760395625*^9}, 
   3.4189352015544357`*^9, {3.4189352809826517`*^9, 3.418935281157464*^9}, {
   3.4190050843295393`*^9, 3.419005105154621*^9}, {3.41900517654276*^9, 
   3.419005176929798*^9}, 3.4196233988517523`*^9, {3.4196235162423773`*^9, 
   3.4196236237892523`*^9}, {3.4196236946017523`*^9, 
   3.4196237579923773`*^9}, {3.4196238226330023`*^9, 
   3.4196238634767523`*^9}, {3.4196238950548773`*^9, 
   3.4196239849767523`*^9}, {3.4196240215861273`*^9, 
   3.4196240342736273`*^9}, {3.4196240667580023`*^9, 
   3.4196241167580023`*^9}, {3.4196241531173773`*^9, 
   3.4196242493361273`*^9}, {3.4196242931486273`*^9, 
   3.4196245394298773`*^9}, {3.4196245814611273`*^9, 
   3.4196245823517523`*^9}, {3.4196246440080023`*^9, 
   3.4196246976642523`*^9}, {3.4196289618048773`*^9, 
   3.4196290003986273`*^9}, {3.4196290549298773`*^9, 
   3.4196291919142523`*^9}, {3.4196292551330023`*^9, 
   3.4196292743517523`*^9}, {3.4196293733205023`*^9, 
   3.4196293808830023`*^9}, {3.4196303038048773`*^9, 3.4196303360080023`*^9}, 
   3.4196305394298773`*^9, {3.4196310448048773`*^9, 3.4196310487580023`*^9}, {
   3.4196332151330023`*^9, 3.4196332358673773`*^9}, {3.4196332766173773`*^9, 
   3.4196332879767523`*^9}, {3.4196361497736273`*^9, 
   3.4196362003673773`*^9}, {3.4196751702476873`*^9, 
   3.4196751904717093`*^9}, {3.4196752308967514`*^9, 
   3.4196753137550364`*^9}, {3.4196753631629763`*^9, 3.419675489782637*^9}, {
   3.4196755199116497`*^9, 3.4196755228269415`*^9}, {3.4196755597996387`*^9, 
   3.4196756007167296`*^9}, {3.4196756344180994`*^9, 
   3.4196760759852514`*^9}, {3.4196762641910706`*^9, 3.41967628028868*^9}, {
   3.419711715046335*^9, 3.41971175840435*^9}, {3.4198889472312307`*^9, 
   3.419888956451102*^9}, {3.4198889907365203`*^9, 3.4198890407269735`*^9}, {
   3.4199367235599747`*^9, 3.4199367322648454`*^9}, {3.4202954775663204`*^9, 
   3.420295477770341*^9}, {3.4203059953233194`*^9, 3.420305999249712*^9}, {
   3.452394669152132*^9, 3.4523946794325323`*^9}}],

Cell[BoxData[
 TagBox[
  StyleBox[
   DynamicModuleBox[{$CellContext`MR$$ = 1.5, $CellContext`VolVol$$ = 
    300, $CellContext`\[Rho]$$ = -0.6, $CellContext`\[Sigma]0$$ = 
    50, $CellContext`\[Sigma]inf$$ = 50, Typeset`show$$ = True, 
    Typeset`bookmarkList$$ = {}, Typeset`bookmarkMode$$ = "Menu", 
    Typeset`animator$$, Typeset`animvar$$ = 1, Typeset`name$$ = 
    "\"untitled\"", Typeset`specs$$ = {{
      Hold[
       Item[
        Grid[{{"original volatility", 
           Manipulator[
            Dynamic[$CellContext`\[Sigma]0$$], {10, 200, 10}, ImageSize -> 
            Tiny], 
           Row[{
             Dynamic[$CellContext`\[Sigma]0$$], "%"}]}, {
          "long-term volatility", 
           Manipulator[
            Dynamic[$CellContext`\[Sigma]inf$$], {10, 200, 10}, ImageSize -> 
            Tiny], 
           Row[{
             Dynamic[$CellContext`\[Sigma]inf$$], "%"}]}, {"correlation", 
           Manipulator[
            Dynamic[$CellContext`\[Rho]$$], {-0.9, 0.9, 0.1}, ImageSize -> 
            Tiny], 
           Row[{
             Dynamic[$CellContext`\[Rho]$$]}]}, {"mean reversion", 
           Manipulator[
            Dynamic[$CellContext`MR$$], {0.1, 10, 0.1}, ImageSize -> Tiny], 
           Dynamic[$CellContext`MR$$]}, {"volatility of volatility", 
           Manipulator[
            Dynamic[$CellContext`VolVol$$], {10, 1000, 10}, ImageSize -> 
            Tiny], 
           Row[{
             Dynamic[$CellContext`VolVol$$], "%"}]}}, 
         Alignment -> {{Right, Left, Right}}, 
         ItemSize -> {{12, 8, 4}, Automatic}]]], 
      Manipulate`Dump`ThisIsNotAControl}, {{
       Hold[$CellContext`\[Sigma]0$$], 40}, 10, 200, 10}, {{
       Hold[$CellContext`\[Sigma]inf$$], 20}, 10, 200, 10}, {{
       Hold[$CellContext`\[Rho]$$], -0.1}, -0.9, 0.9, 0.1}, {{
       Hold[$CellContext`MR$$], 1.5}, 0.1, 10, 0.1}, {{
       Hold[$CellContext`VolVol$$], 80}, 10, 1000, 10}}, Typeset`size$$ = {
    600., {298., 302.}}, Typeset`update$$ = 0, Typeset`initDone$$, 
    Typeset`skipInitDone$$ = False, $CellContext`\[Sigma]0$13970$$ = 
    0, $CellContext`\[Sigma]inf$13971$$ = 0, $CellContext`\[Rho]$13972$$ = 
    0, $CellContext`MR$13973$$ = 0, $CellContext`VolVol$13974$$ = 0}, 
    DynamicBox[Manipulate`ManipulateBoxes[
     1, StandardForm, 
      "Variables" :> {$CellContext`MR$$ = 1.5, $CellContext`VolVol$$ = 
        80, $CellContext`\[Rho]$$ = -0.1, $CellContext`\[Sigma]0$$ = 
        40, $CellContext`\[Sigma]inf$$ = 20}, "ControllerVariables" :> {
        Hold[$CellContext`\[Sigma]0$$, $CellContext`\[Sigma]0$13970$$, 0], 
        Hold[$CellContext`\[Sigma]inf$$, $CellContext`\[Sigma]inf$13971$$, 0], 
        Hold[$CellContext`\[Rho]$$, $CellContext`\[Rho]$13972$$, 0], 
        Hold[$CellContext`MR$$, $CellContext`MR$13973$$, 0], 
        Hold[$CellContext`VolVol$$, $CellContext`VolVol$13974$$, 0]}, 
      "OtherVariables" :> {
       Typeset`show$$, Typeset`bookmarkList$$, Typeset`bookmarkMode$$, 
        Typeset`animator$$, Typeset`animvar$$, Typeset`name$$, 
        Typeset`specs$$, Typeset`size$$, Typeset`update$$, Typeset`initDone$$,
         Typeset`skipInitDone$$}, "Body" :> Quiet[
        Module[{$CellContext`i$, $CellContext`j$, $CellContext`x$, \
$CellContext`k$, $CellContext`T$, $CellContext`Price$, $CellContext`pr$, \
$CellContext`Volatility$, $CellContext`vol$}, $CellContext`Price$ = 
          Array[$CellContext`pr$, {$CellContext`m, $CellContext`t}]; \
$CellContext`Volatility$ = Array[$CellContext`vol$, $CellContext`size]; 
         For[$CellContext`i$ = 1, $CellContext`i$ < $CellContext`t + 1, 
           
           Increment[$CellContext`i$], $CellContext`T$ = 
            Part[$CellContext`Tenors, $CellContext`i$]/12; 
           For[$CellContext`j$ = 1, $CellContext`j$ < $CellContext`m + 1, 
             
             Increment[$CellContext`j$], $CellContext`k$ = $CellContext`S/
               Part[$CellContext`Moneyness, $CellContext`j$]; \
$CellContext`pr$[$CellContext`i$, $CellContext`j$] = \
$CellContext`HestonCallLewis[$CellContext`S, $CellContext`k$, \
$CellContext`T$, $CellContext`r, $CellContext`d, ($CellContext`\[Sigma]0$$/
                 100)^2, $CellContext`MR$$, $CellContext`\[Rho]$$, \
($CellContext`\[Sigma]inf$$/100)^2, $CellContext`VolVol$$/
                100]; $CellContext`x$ = $CellContext`ImpliedVolCall[
                $CellContext`pr$[$CellContext`i$, $CellContext`j$], \
$CellContext`S, $CellContext`k$, $CellContext`T$, $CellContext`r, \
$CellContext`d, 0.0001]; 
             If[$CellContext`x$ != 
               1.*^-6, $CellContext`vol$[($CellContext`i$ - 
                   1) $CellContext`m + $CellContext`j$] = {
                 Part[$CellContext`Tenors, $CellContext`i$], 
                 Part[$CellContext`Moneyness, $CellContext`j$], 
                 100 $CellContext`x$}, $CellContext`vol$[($CellContext`i$ - 
                   1) $CellContext`m + $CellContext`j$] = {Null, Null, Null}]; 
             Null]; Null]; 
         ListPlot3D[$CellContext`Volatility$, PlotRange -> {Full, Full, Full},
            PlotLabel -> "Implied Volatility Surface", 
           AxesLabel -> {"time to expiry\nin months", "moneyness", None}, 
           ImageSize -> {600, 600}, BoxRatios -> {1, 1, 0.6}, ImagePadding -> 
           25]]], "Specifications" :> {
        Item[
         Grid[{{"original volatility", 
            Manipulator[
             Dynamic[$CellContext`\[Sigma]0$$], {10, 200, 10}, ImageSize -> 
             Tiny], 
            Row[{
              Dynamic[$CellContext`\[Sigma]0$$], "%"}]}, {
           "long-term volatility", 
            Manipulator[
             Dynamic[$CellContext`\[Sigma]inf$$], {10, 200, 10}, ImageSize -> 
             Tiny], 
            Row[{
              Dynamic[$CellContext`\[Sigma]inf$$], "%"}]}, {"correlation", 
            Manipulator[
             Dynamic[$CellContext`\[Rho]$$], {-0.9, 0.9, 0.1}, ImageSize -> 
             Tiny], 
            Row[{
              Dynamic[$CellContext`\[Rho]$$]}]}, {"mean reversion", 
            Manipulator[
             Dynamic[$CellContext`MR$$], {0.1, 10, 0.1}, ImageSize -> Tiny], 
            Dynamic[$CellContext`MR$$]}, {"volatility of volatility", 
            Manipulator[
             Dynamic[$CellContext`VolVol$$], {10, 1000, 10}, ImageSize -> 
             Tiny], 
            Row[{
              Dynamic[$CellContext`VolVol$$], "%"}]}}, 
          Alignment -> {{Right, Left, Right}}, 
          ItemSize -> {{12, 8, 4}, Automatic}]], {{$CellContext`\[Sigma]0$$, 
          40}, 10, 200, 10, ControlType -> 
         None}, {{$CellContext`\[Sigma]inf$$, 20}, 10, 200, 10, ControlType -> 
         None}, {{$CellContext`\[Rho]$$, -0.1}, -0.9, 0.9, 0.1, ControlType -> 
         None}, {{$CellContext`MR$$, 1.5}, 0.1, 10, 0.1, ControlType -> 
         None}, {{$CellContext`VolVol$$, 80}, 10, 1000, 10, ControlType -> 
         None}}, "Options" :> {
       SynchronousUpdating -> False, ControlPlacement -> Left, TrackedSymbols -> 
        Manipulate}, "DefaultOptions" :> {}],
     ImageSizeCache->{847., {325.34375, 330.65625}},
     SingleEvaluation->True],
    Deinitialization:>None,
    DynamicModuleValues:>{},
    Initialization:>({$CellContext`m = 7, $CellContext`t = 
       8, $CellContext`size = 
       56, $CellContext`Tenors = {1, 3, 6, 12, 18, 24, 30, 
        36}, $CellContext`S = 
       100, $CellContext`Moneyness = {
        0.6, 0.75, 0.9, 1, 1.1, 1.25, 1.4}, $CellContext`HestonCallLewis[
         Pattern[$CellContext`S, 
          Blank[]], 
         Pattern[K, 
          Blank[]], 
         Pattern[$CellContext`\[Tau], 
          Blank[]], 
         Pattern[$CellContext`r, 
          Blank[]], 
         Pattern[$CellContext`d, 
          Blank[]], 
         Pattern[$CellContext`V0, 
          Blank[]], 
         Pattern[$CellContext`MR, 
          Blank[]], 
         Pattern[$CellContext`\[Rho], 
          Blank[]], 
         Pattern[$CellContext`Vinf, 
          Blank[]], 
         Pattern[$CellContext`VolVol, 
          Blank[]]] := 
       Module[{Integral, $CellContext`X, $CellContext`\[Theta], $CellContext`\
\[Xi], $CellContext`\[Omega], $CellContext`V, $CellContext`k}, $CellContext`X = 
          Log[$CellContext`S/
             K] + ($CellContext`r - $CellContext`d) $CellContext`\[Tau]; \
$CellContext`k = $CellContext`kre + I $CellContext`kim; 
         Integral = (1/Pi) NIntegrate[
             Re[
             Exp[((-I) $CellContext`k) $CellContext`X] \
($CellContext`Hhat[$CellContext`k, $CellContext`V0, $CellContext`\[Tau], \
$CellContext`MR, $CellContext`VolVol, $CellContext`Vinf $CellContext`MR, \
$CellContext`\[Rho]]/($CellContext`k^2 - 
               I $CellContext`k))], {$CellContext`kre, 
              0, $CellContext`\[CapitalLambda]}, 
             Method -> {"LobattoKronrodRule", "Points" -> 25}, PrecisionGoal -> 
             4]; $CellContext`S Exp[(-$CellContext`d) $CellContext`\[Tau]] - (
           K Exp[(-$CellContext`r) $CellContext`\[Tau]]) 
          Integral], $CellContext`r = 0.03, $CellContext`d = 
       0., $CellContext`kim = 0.6, $CellContext`Hhat[
         Pattern[$CellContext`k, 
          Blank[]], 
         Pattern[$CellContext`V, 
          Blank[]], 
         Pattern[$CellContext`\[Tau], 
          Blank[]], 
         Pattern[$CellContext`\[Theta], 
          Blank[]], 
         Pattern[$CellContext`\[Xi], 
          Blank[]], 
         Pattern[$CellContext`\[Omega], 
          Blank[]], 
         Pattern[$CellContext`\[Rho], 
          Blank[]]] := 
       Module[{$CellContext`f1, $CellContext`f2, $CellContext`d, \
$CellContext`g, $CellContext`h, $CellContext`ttheta, $CellContext`t, \
$CellContext`tomega, $CellContext`tc}, $CellContext`c = (1/
            2) ($CellContext`k^2 - 
            I $CellContext`k); $CellContext`t = ((1/
             2) $CellContext`\[Xi]^2) $CellContext`\[Tau]; \
$CellContext`tomega = (
            2/$CellContext`\[Xi]^2) $CellContext`\[Omega]; $CellContext`tc = (
            2/$CellContext`\[Xi]^2) $CellContext`c; $CellContext`ttheta = (
            2/$CellContext`\[Xi]^2) (((
               I $CellContext`k) $CellContext`\[Rho]) $CellContext`\[Xi] + \
$CellContext`\[Theta]); $CellContext`d = 
          Sqrt[$CellContext`ttheta^2 + 4 $CellContext`tc]; $CellContext`g = (
            1/2) ($CellContext`ttheta + $CellContext`d); $CellContext`h = \
($CellContext`ttheta + $CellContext`d)/($CellContext`ttheta - $CellContext`d); \
$CellContext`f1 = $CellContext`tomega ($CellContext`t $CellContext`g - 
            Log[(1 - $CellContext`h Exp[$CellContext`d $CellContext`t])/(
             1 - $CellContext`h)]); $CellContext`f2 = $CellContext`g ((1 - 
             Exp[$CellContext`d $CellContext`t])/(1 - $CellContext`h 
            Exp[$CellContext`d $CellContext`t])); 
         Exp[$CellContext`f1 + $CellContext`f2 $CellContext`V]], \
$CellContext`c = ((-I) (0.6 I + $CellContext`kre) + (
           0.6 I + $CellContext`kre)^2)/2, $CellContext`\[CapitalLambda] = 
       125, $CellContext`ImpliedVolCall = CompiledFunction[{
          Blank[Real], 
          Blank[Real], 
          Blank[Real], 
          Blank[Real], 
          Blank[Real], 
          Blank[Real], 
          Blank[Real]}, {{3, 0, 0}, {3, 0, 1}, {3, 0, 2}, {3, 0, 3}, {3, 0, 
         4}, {3, 0, 5}, {3, 0, 6}, {3, 0, 14}}, {3, 1, 17, 0, 
         0}, {{1, 5}, {8, 0.8, 7}, {8, 0., 8}, {24, 5, 9}, {21, 9, 3, 9}, {93,
           34, 3, 0, 9, 3, 0, 10}, {21, 1, 10, 9}, {24, 4, 10}, {21, 10, 3, 
          10}, {93, 34, 3, 0, 10, 3, 0, 11}, {21, 2, 11, 10}, {24, 10, 11}, {
          18, 9, 11, 9}, {7, 0, 0}, {15, 0, 0, 11}, {37, 9, 11, 0}, {3, 0, 
          4}, {8, 0., 11}, {12, 11, 9}, {4, 1}, {37, 0, 9, 0}, {3, 0, 4}, {
          8, 1.*^-6, 11}, {12, 11, 14}, {4, 34}, {12, 7, 10}, {54, 
           
           Function[{$CellContext`p, $CellContext`S, 
             K, $CellContext`T, $CellContext`r, $CellContext`d, \
$CellContext`precision}, 
            $CellContext`BSCall[$CellContext`S, 
             K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]], {$CellContext`Vol1, 3, 0, 10, Module}, 3, 0, 0, 3, 0, 1, 3, 
           0, 2, 3, 0, 3, 3, 0, 4, 3, 0, 5, 3, 0, 6, 3, 0, 12}, {37, 12, 0, 
          1}, {3, 1, 8}, {54, 
           
           Function[{$CellContext`p, $CellContext`S, 
             K, $CellContext`T, $CellContext`r, $CellContext`d, \
$CellContext`precision}, 
            $CellContext`BSCall[$CellContext`S, 
             K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]], {$CellContext`Vol1, 3, 0, 10, Module}, 3, 0, 0, 3, 0, 1, 3, 
           0, 2, 3, 0, 3, 3, 0, 4, 3, 0, 5, 3, 0, 6, 3, 0, 13}, {12, 13, 
          12}, {7, 2, 0}, {15, 1, 0, 13}, {21, 13, 10, 13}, {12, 13, 10}, {
          4, -8}, {12, 10, 7}, {54, 
           
           Function[{$CellContext`p, $CellContext`S, 
             K, $CellContext`T, $CellContext`r, $CellContext`d, \
$CellContext`precision}, 
            $CellContext`BSCall[$CellContext`S, 
             K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]], {$CellContext`Vol1, 3, 0, 10, Module}, 3, 0, 0, 3, 0, 1, 3, 
           0, 2, 3, 0, 3, 3, 0, 4, 3, 0, 5, 3, 0, 6, 3, 0, 13}, {24, 0, 14}, {
          18, 13, 14, 15}, {93, 36, 3, 0, 15, 3, 0, 14}, {37, 6, 14, 1}, {3, 
          1, 15}, {18, 8, 7, 14}, {7, 1, 0}, {8, 2., 15}, {94, 260, 2, 0, 0, 
          3, 0, 15, 3, 0, 16}, {21, 14, 16, 14}, {12, 14, 10}, {54, 
           
           Function[{$CellContext`p, $CellContext`S, 
             K, $CellContext`T, $CellContext`r, $CellContext`d, \
$CellContext`precision}, 
            $CellContext`BSCall[$CellContext`S, 
             K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]], {$CellContext`Vol1, 3, 0, 10, Module}, 3, 0, 0, 3, 0, 1, 3, 
           0, 2, 3, 0, 3, 3, 0, 4, 3, 0, 5, 3, 0, 6, 3, 0, 14}, {12, 14, 
          13}, {37, 13, 0, 2}, {3, 2, 3}, {12, 10, 8}, {4, 2}, {12, 10, 7}, {
          4, -18}, {12, 10, 14}, {2}}, 
         Function[{$CellContext`p, $CellContext`S, 
           K, $CellContext`T, $CellContext`r, $CellContext`d, \
$CellContext`precision}, 
          
          Module[{$CellContext`Vol0, $CellContext`Vol1, $CellContext`Vol2, \
$CellContext`Price0, $CellContext`Price1, $CellContext`Price2}, \
$CellContext`Vol2 = 0.8; $CellContext`Vol0 = 
            0.; $CellContext`Price0 = $CellContext`S 
              Exp[(-$CellContext`d) $CellContext`T] - K 
             Exp[(-$CellContext`r) $CellContext`T]; 
           If[$CellContext`Price0 < 0, $CellContext`Price0 = 0.]; 
           If[$CellContext`p < $CellContext`Price0, 
             1.*^-6, $CellContext`Vol1 = $CellContext`Vol2; \
$CellContext`Price2 = $CellContext`BSCall[$CellContext`S, 
                K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]; 
             While[$CellContext`Price2 < $CellContext`p, $CellContext`Price2 = \
$CellContext`BSCall[$CellContext`S, 
                  K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]; $CellContext`Vol1 = 2 $CellContext`Vol1; 
               Null]; $CellContext`Vol2 = $CellContext`Vol1; \
$CellContext`Price1 = $CellContext`BSCall[$CellContext`S, 
                K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]; 
             While[Abs[$CellContext`Price1 - $CellContext`p] > \
$CellContext`precision, $CellContext`Vol1 = ($CellContext`Vol0 + \
$CellContext`Vol2)/
                 2.; $CellContext`Price1 = $CellContext`BSCall[$CellContext`S,
                   K, $CellContext`Vol1, $CellContext`T, $CellContext`r, \
$CellContext`d]; 
               If[$CellContext`Price1 < $CellContext`p, $CellContext`Vol0 = \
$CellContext`Vol1, $CellContext`Vol2 = $CellContext`Vol1]; 
               Null]; $CellContext`Vol1]]], Evaluate], $CellContext`BSCall = 
       CompiledFunction[{
          Blank[Real], 
          Blank[Real], 
          Blank[Real], 
          Blank[Real], 
          Blank[Real], 
          Blank[Real]}, {{3, 0, 0}, {3, 0, 1}, {3, 0, 2}, {3, 0, 3}, {3, 0, 
         4}, {3, 0, 5}, {3, 0, 11}}, {0, 2, 12, 0, 
         0}, {{1, 5}, {7, 1, 0}, {94, 260, 2, 0, 0, 3, 0, 1, 3, 0, 6}, {21, 0,
           6, 7}, {93, 35, 3, 0, 7, 3, 0, 6}, {24, 5, 7}, {18, 4, 7, 8}, {21, 
          8, 3, 8}, {7, 2, 0}, {94, 264, 3, 0, 2, 2, 0, 0, 3, 0, 7}, {7, 1, 
          0}, {7, 2, 1}, {15, 0, 0, 9}, {15, 0, 1, 10}, {94, 260, 3, 0, 9, 3, 
          0, 10, 3, 0, 11}, {21, 3, 11, 9}, {21, 7, 9, 7}, {18, 6, 8, 7, 6}, {
          7, 1, 0}, {93, 55, 3, 0, 3, 3, 0, 8}, {21, 2, 8, 7}, {94, 260, 2, 0,
           0, 3, 0, 7, 3, 0, 8}, {21, 6, 8, 6}, {93, 55, 3, 0, 3, 3, 0, 8}, {
          21, 2, 8, 7}, {24, 7, 8}, {18, 6, 8, 7}, {24, 5, 8}, {21, 8, 3, 
          8}, {93, 34, 3, 0, 8, 3, 0, 9}, {54, 
           
           Function[{$CellContext`S, 
             K, $CellContext`\[Sigma], $CellContext`T, $CellContext`r, \
$CellContext`d}, 
            $CellContext`Ncdf[$CellContext`d1]], {$CellContext`d1, 3, 0, 6, 
            Module}, 3, 0, 0, 3, 0, 1, 3, 0, 2, 3, 0, 3, 3, 0, 4, 3, 0, 5, 3, 
           0, 8}, {21, 0, 9, 8, 11}, {24, 4, 9}, {21, 9, 3, 9}, {93, 34, 3, 0,
           9, 3, 0, 8}, {54, 
           
           Function[{$CellContext`S, 
             K, $CellContext`\[Sigma], $CellContext`T, $CellContext`r, \
$CellContext`d}, 
            $CellContext`Ncdf[$CellContext`d2]], {$CellContext`d2, 3, 0, 7, 
            Module}, 3, 0, 0, 3, 0, 1, 3, 0, 2, 3, 0, 3, 3, 0, 4, 3, 0, 5, 3, 
           0, 9}, {21, 1, 8, 9, 10}, {24, 10, 8}, {18, 11, 8, 11}, {2}}, 
         Function[{$CellContext`S, 
           K, $CellContext`\[Sigma], $CellContext`T, $CellContext`r, \
$CellContext`d}, 
          
          Module[{$CellContext`d1, $CellContext`d2}, $CellContext`d1 = (
              Log[$CellContext`S/
                K] + ($CellContext`r - $CellContext`d) $CellContext`T + \
$CellContext`\[Sigma]^2 ($CellContext`T/2))/($CellContext`\[Sigma] 
             Sqrt[$CellContext`T]); $CellContext`d2 = $CellContext`d1 - \
$CellContext`\[Sigma] 
             Sqrt[$CellContext`T]; ($CellContext`S \
$CellContext`Ncdf[$CellContext`d1])/E^($CellContext`d $CellContext`T) - (
             K $CellContext`Ncdf[$CellContext`d2])/
            E^($CellContext`r $CellContext`T)]], Evaluate], $CellContext`Ncdf = 
       CompiledFunction[{
          Blank[Real]}, {{3, 0, 0}, {3, 0, 1}}, {0, 2, 5, 0, 
         0}, {{1, 5}, {7, 1, 0}, {7, 2, 1}, {15, 0, 1, 1}, {93, 55, 3, 0, 1, 
          3, 0, 2}, {94, 260, 2, 0, 0, 3, 0, 2, 3, 0, 1}, {21, 0, 1, 2}, {
          55, Erf, 3, 0, 2, 3, 0, 1}, {7, 1, 0}, {15, 0, 0, 2}, {18, 1, 2, 
          1}, {7, 1, 0}, {7, 2, 1}, {15, 0, 0, 2}, {15, 0, 1, 3}, {94, 260, 3,
           0, 2, 3, 0, 3, 3, 0, 4}, {21, 1, 4, 1}, {2}}, 
         Function[{$CellContext`x}, (Erf[$CellContext`x/Sqrt[2]] + 1)/2], 
         Evaluate], Attributes[PlotRange] = {ReadProtected}}; 
     Typeset`initDone$$ = True),
    SynchronousInitialization->True,
    UnsavedVariables:>{Typeset`initDone$$},
    UntrackedVariables:>{Typeset`size$$}], "Manipulate",
   Deployed->True,
   StripOnInput->False],
  Manipulate`InterpretManipulate[1]]], "Output",
 CellChangeTimes->{3.4523946856413317`*^9}]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["THIS NOTEBOOK IS THE SOURCE CODE FROM", "Text",
 CellFrame->{{0, 0}, {0, 1}},
 CellMargins->{{48, 10}, {4, 28}},
 CellGroupingRules->{"SectionGrouping", 25},
 CellFrameMargins->{{48, 48}, {6, 5}},
 CellFrameColor->RGBColor[0.691905, 0.790311, 0.300252],
 FontFamily->"Helvetica",
 FontSize->10,
 FontWeight->"Bold",
 FontColor->RGBColor[0.691905, 0.790311, 0.300252]],

Cell[TextData[{
 "\"",
 ButtonBox["Volatility Surface in the Heston Model",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/VolatilitySurfaceInTheHestonModel/\
"], None},
  ButtonNote->
   "http://demonstrations.wolfram.com/VolatilitySurfaceInTheHestonModel/"],
 "\"",
 " from ",
 ButtonBox["The Wolfram Demonstrations Project",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/"], None},
  ButtonNote->"http://demonstrations.wolfram.com/"],
 "\[ParagraphSeparator]\[NonBreakingSpace]",
 ButtonBox["http://demonstrations.wolfram.com/\
VolatilitySurfaceInTheHestonModel/",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/VolatilitySurfaceInTheHestonModel/\
"], None},
  ButtonNote->
   "http://demonstrations.wolfram.com/VolatilitySurfaceInTheHestonModel/"]
}], "Text",
 CellMargins->{{48, Inherited}, {0, Inherited}},
 FontFamily->"Verdana",
 FontSize->10,
 FontColor->GrayLevel[0.5]]
}, Open  ]],

Cell[CellGroupData[{

Cell[" ", "Text",
 CellFrame->{{0, 0}, {0, 1}},
 CellMargins->{{48, 10}, {4, 28}},
 CellGroupingRules->{"SectionGrouping", 25},
 CellFrameMargins->{{48, 48}, {6, 5}},
 CellFrameColor->RGBColor[0.691905, 0.790311, 0.300252],
 FontFamily->"Helvetica",
 FontSize->10,
 FontWeight->"Bold",
 FontColor->RGBColor[0.691905, 0.790311, 0.300252]],

Cell[TextData[{
 "Contributed by: ",
 ButtonBox["Slava Solganik",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/author.html?author=Slava+Solganik"]\
, None}]
}], "Text",
 CellDingbat->"\[FilledSmallSquare]",
 CellMargins->{{66, 48}, {2, 4}},
 FontFamily->"Verdana",
 FontSize->10,
 FontColor->GrayLevel[0.6]],

Cell[TextData[{
 "A full-function Wolfram ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " system (Version 6 or higher) is required to edit this notebook.\n",
 StyleBox[ButtonBox["GET WOLFRAM MATHEMATICA \[RightGuillemet]",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://www.wolfram.com/products/mathematica/"], None},
  ButtonNote->"http://www.wolfram.com/products/mathematica/"],
  FontFamily->"Helvetica",
  FontWeight->"Bold",
  FontSlant->"Italic",
  FontColor->RGBColor[1, 0.42, 0]]
}], "Text",
 CellFrame->True,
 CellMargins->{{48, 68}, {8, 28}},
 CellFrameMargins->12,
 CellFrameColor->RGBColor[0.865507, 0.90634, 0.680751],
 CellChangeTimes->{3.3750111182355957`*^9},
 ParagraphSpacing->{1., 1.},
 FontFamily->"Verdana",
 FontSize->10,
 FontColor->GrayLevel[0.411765],
 Background->RGBColor[0.986023, 0.991363, 0.969818]],

Cell[TextData[{
 "\[Copyright] ",
 StyleBox[ButtonBox["The Wolfram Demonstrations Project & Contributors",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/"], None},
  ButtonNote->"http://demonstrations.wolfram.com/"],
  FontColor->GrayLevel[0.6]],
 "\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
 StyleBox[ButtonBox["Terms of Use",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/termsofuse.html"], None},
  ButtonNote->"http://demonstrations.wolfram.com/termsofuse.html"],
  FontColor->GrayLevel[0.6]],
 "\[ThickSpace]\[ThickSpace]\[ThickSpace]|\[ThickSpace]\[ThickSpace]\
\[ThickSpace]",
 StyleBox[ButtonBox["Make a new version of this Demonstration \
\[RightGuillemet]",
  BaseStyle->"Hyperlink",
  ButtonData->{
    URL["http://demonstrations.wolfram.com/participate/upload.jsp?id=\
VolatilitySurfaceInTheHestonModel"], None},
  ButtonNote->None],
  FontColor->GrayLevel[0.6]]
}], "Text",
 CellFrame->{{0, 0}, {0, 0.5}},
 CellMargins->{{48, 10}, {20, 50}},
 CellFrameMargins->{{6, 0}, {6, 6}},
 CellFrameColor->GrayLevel[0.6],
 FontFamily->"Verdana",
 FontSize->9,
 FontColor->GrayLevel[0.6]]
}, Open  ]]
},
AutoGeneratedPackage->None,
Editable->True,
Saveable->False,
ScreenStyleEnvironment->"Working",
WindowSize->{1220, 756},
WindowMargins->{{0, Automatic}, {Automatic, 0}},
WindowElements->{
 "StatusArea", "MemoryMonitor", "MagnificationPopUp", "VerticalScrollBar", 
  "MenuBar"},
WindowTitle->"Volatility Surface in the Heston Model - Source",
DockedCells->{},
CellContext->Notebook,
FrontEndVersion->"7.0 for Mac OS X x86 (32-bit) (November 10, 2008)",
StyleDefinitions->"Default.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[628, 23, 230, 4, 67, "Section"],
Cell[861, 29, 11865, 338, 618, "Input",
 InitializationCell->True,
 CellID->92223448],
Cell[CellGroupData[{
Cell[12751, 371, 11255, 261, 388, "Input"],
Cell[24009, 634, 19189, 369, 673, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[43247, 1009, 373, 9, 50, "Text",
 CellGroupingRules->{"SectionGrouping", 25}],
Cell[43623, 1020, 994, 29, 35, "Text"]
}, Open  ]],
Cell[CellGroupData[{
Cell[44654, 1054, 337, 9, 50, "Text",
 CellGroupingRules->{"SectionGrouping", 25}],
Cell[44994, 1065, 347, 12, 18, "Text"],
Cell[45344, 1079, 847, 24, 99, "Text"],
Cell[46194, 1105, 1209, 33, 88, "Text"]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
(* NotebookSignature 7RjKm6UeYbM0NA01r85wj#Q6 *)
